/*
  The client side simply sends a hugh number of requests to the
  chosen server as fast as possible, spawing one thread to do the
  sending and another to receive the replies.

  Nothing interesting here.


cc -o client client.c -L. -R. -g -lpthread -lthread -pthread_extensions -lposix4 -lnsl -lsocket
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <pthread.h>
#include "thread_extensions.h"


/* 127.0.0.1 is the loopback address.  You may change it. */
const char *SERV_HOST_ADDR =   "127.0.0.1";
int TCP_PORT = 6500;
int N_SEGMENTS = 10000;


/* Send out 10,000 segments */
void *talk(void *arg)
{int i, j, sockfd = (int) arg;
 char obuf[100], ibuf[100];
 char *name = thread_name(pthread_self());

 printf("[%s] \tSending %d segments...\n", name, N_SEGMENTS);
 for (j=1; j<N_SEGMENTS+1; j++)
   {sprintf(obuf, "DATA SEGMENT %d [%s].", j, name);
    write(sockfd, obuf, 50);
    DEBUG(printf("[%s] sending: \"%s\"\n", name, obuf));
    if (j%1000 == 0) printf("[%s] \tSent %d segments.\n", name, j);
  }
 write(sockfd, "End", 4);
 printf("[%s] \tDone sending.\n", name);
}

/* Listen to segments until "End" */
void *hear(void *arg)
{int i, j, sockfd = (int) arg;
 struct sockaddr_in serv_addr;
 char obuf[100], ibuf[100];
 char *name = thread_name(pthread_self());

 printf("[%s] \tReceiving segments...\n", name);
 for (j=1; 1; j++)
   {read(sockfd, ibuf, 50);
    if (strcmp(ibuf, "End") == 0) break;
    if (j%1000 == 0) printf("[%s] \tReceived %d segments.\n", name, j);
    DEBUG(printf("[%s] received: \"%s\"\n", name, ibuf));
  }
 close(sockfd);
 printf("[%s] \tDone receiving.\n", name);
}



main(int argc, char **argv)
{int i, j, sockfd, N_SOCKETS = 1;
 pthread_attr_t attr;
 struct sockaddr_in serv_addr;
 pthread_t tid;

 if (argc >= 2) TCP_PORT    = atoi(argv[1]);
 if (argc >= 3) N_SOCKETS   = atoi(argv[2]);
 printf("TCP_PORT = %d N_SOCKETS (10k msg each) = %d\n", TCP_PORT, N_SOCKETS);

 PTHREAD_ATTR_INIT(&attr);
 pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
 pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);

 memset((char *) &serv_addr, 0, sizeof(serv_addr));
 serv_addr.sin_family = AF_INET;
 serv_addr.sin_addr.s_addr = inet_addr(SERV_HOST_ADDR);
 serv_addr.sin_port = htons(TCP_PORT);


 for (i=0; i < N_SOCKETS; i++)
   {if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
      {perror("clientsoc: can't open stream socket"); exit(0);}
    if (connect(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0)
      perror("clientsoc: can't connect to server"), exit(0);

    PTHREAD_CREATE(&tid, &attr, talk, (void *) sockfd);
    PTHREAD_CREATE(&tid, &attr, hear, (void *) sockfd);
    }

 pthread_exit(NULL);
}

